@@ -1,6 +1,8 @@
 {-# LANGUAGE CPP #-}
 module GhcUtil (withGhc) where
 
+import           Imports
+
 import           GHC.Paths (libdir)
 import           GHC
 #if __GLASGOW_HASKELL__ < 900
@@ -23,10 +25,6 @@ import           GHC.Utils.Monad (liftIO)
 
 import           System.Exit (exitFailure)
 
-#if __GLASGOW_HASKELL__ < 801
-import           StaticFlags (discardStaticFlags)
-#endif
-
 -- Catch GHC source errors, print them and exit.
 handleSrcErrors :: Ghc a -> Ghc a
 handleSrcErrors action' = flip handleSourceError action' $ \err -> do
@@ -42,11 +40,7 @@ withGhc flags action = do
     handleDynamicFlags flags_ >>= handleSrcErrors . action
 
 handleStaticFlags :: [String] -> IO [Located String]
-#if __GLASGOW_HASKELL__ < 801
-handleStaticFlags flags = return $ map noLoc $ discardStaticFlags flags
-#else
 handleStaticFlags flags = return $ map noLoc $ flags
-#endif
 
 handleDynamicFlags :: GhcMonad m => [Located String] -> m [String]
 handleDynamicFlags flags = do
@@ -69,7 +63,9 @@ handleDynamicFlags flags = do
 
 setHaddockMode :: DynFlags -> DynFlags
 setHaddockMode dynflags = (gopt_set dynflags Opt_Haddock) {
-#if __GLASGOW_HASKELL__ >= 901
+#if __GLASGOW_HASKELL__ >= 906
+      backend   = noBackend
+#elif __GLASGOW_HASKELL__ >= 901
       backend   = NoBackend
 #else
       hscTarget = HscNothing
